from django.db import models
from django.contrib.auth.models import User


class Studienabschnitt(models.Model):
    """
    z.B. "1. Semester", "2. Semester", "Physikum", "STEX", "PJ", ...
    """
    bezeichnung = models.CharField(max_length=100)
    sortierung = models.PositiveSmallIntegerField()

    def __str__(self):
        return self.bezeichnung

    class Meta:
        verbose_name = 'Studienabschnitt/Semester'
        verbose_name_plural = 'Studienabschnitte/Semester'
        ordering = ['sortierung']


class Studiengang(models.Model):
    """
    z.B. "Humanmedizin", "Zahnmedizin", ...
    """
    bezeichnung = models.CharField(max_length=100)
    studienabschnitt = models.ManyToManyField(
        Studienabschnitt,
        blank=True,
        related_name='studiengang'
    )

    def __str__(self):
        return self.bezeichnung

    class Meta:
        verbose_name = 'Studiengang'
        verbose_name_plural = 'Studiengänge'
        ordering = ['bezeichnung']


class Gender(models.Model):
    bezeichnung = models.CharField(max_length=100)
    # z.B. "in" für "weiblich" -> "Studentin"
    endung = models.CharField(
        max_length=8,
        blank=True,
    )

    def __str__(self):
        return self.bezeichnung

    class Meta:
        verbose_name = 'Gender/Geschlecht'
        verbose_name_plural = 'Gender/Geschlechter'
        ordering = ['bezeichnung']


class FachschaftUser(models.Model):
    """
    Extends django.contrib.auth.models.User
    FachschaftUser = "StudentInnen allgemein"
    """
    user = models.OneToOneField(
        User,
        on_delete=models.PROTECT,
        related_name='fachschaftuser',
    )
    gender = models.ForeignKey(Gender, on_delete=models.PROTECT)
    studienabschnitt = models.ForeignKey(
        Studienabschnitt,
        on_delete=models.PROTECT,
    )
    studiengang = models.ForeignKey(
        Studiengang,
        on_delete=models.PROTECT,
    )
    nickname = models.CharField(
        max_length=100,
        unique=True,
    )

    def __str__(self):
        """
        returns "[Vorname] [Nachname]"
        """
        return self.user.get_full_name()

    class Meta:
        verbose_name = 'StudentIn'
        verbose_name_plural = 'StudentInnen'
        ordering = ['nickname']



from django.contrib import admin

from .models import (
    FachschaftUser,
    Gender,
    Studienabschnitt,
    Studiengang,
)


class FachschaftUserAdmin(admin.ModelAdmin):
    model = FachschaftUser
    fields = [
        'nickname',
        'user',
        'gender',
        'studiengang',
        'studienabschnitt',
    ]
    list_display = (
        'nickname',
        '__str__',
        'gender',
        'studiengang',
        'studienabschnitt',
    )
    search_fields = [
        'nickname',
        'user',
    ]
    list_filter = (
        'gender',
        'studiengang',
        'studienabschnitt',
    )


class GenderAdmin(admin.ModelAdmin):
    model = Gender
    fields = ['bezeichnung', 'endung']


class StudiengangAdmin(admin.ModelAdmin):
    model = Studiengang
    fields = ['bezeichnung']
    # If you don't specify this, you will get a multiple select widget:
    filter_horizontal = ('studienabschnitt',)


admin.site.register(FachschaftUser, FachschaftUserAdmin)
admin.site.register(Gender, GenderAdmin)
admin.site.register(Studienabschnitt)
admin.site.register(Studiengang, StudiengangAdmin)



from django.urls import path
from . import views

app_name = 'fsmedhro_core'

urlpatterns = [
    path('edit/', views.FachschaftUserEdit.as_view(), name='edit'),
    path('', views.FachschaftUserDetail.as_view(), name='detail'),
]



from django.contrib.auth.decorators import login_required
from django.core.exceptions import ObjectDoesNotExist
from django.shortcuts import redirect, render
from django.utils.decorators import method_decorator
from django.views import View

from .forms import FachschaftUserForm


@method_decorator(login_required, name='dispatch')
class FachschaftUserEdit(View):
    def get(self, request):
        try:
            # FachschaftUser bereits vorhanden?
            form = FachschaftUserForm(instance=request.user.fachschaftuser)
        except ObjectDoesNotExist:
            form = FachschaftUserForm()

        context = {
            'form': form,
        }

        return render(request, 'fsmedhro_core/user_edit.html', context)

    def post(self, request):
        try:
            form = FachschaftUserForm(
                data=request.POST,
                instance=request.user.fachschaftuser,
            )
        except ObjectDoesNotExist:
            form = FachschaftUserForm(
                data=request.POST,
            )

        if form.is_valid():
            fachschaftuser = form.save(commit=False)
            fachschaftuser.user = request.user
            fachschaftuser.save()

        return redirect('fsmedhro_core:detail')


@method_decorator(login_required, name='dispatch')
class FachschaftUserDetail(View):
    def get(self, request):
        try:
            fachschaftuser = request.user.fachschaftuser
        except ObjectDoesNotExist:
            return redirect('fsmedhro_core:edit')

        context = {
            'fachschaftuser': fachschaftuser,
        }

        return render(request, 'fsmedhro_core/user_detail.html', context)



# Generated by Django 2.1.8 on 2019-05-04 14:18

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='FachschaftUser',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('nickname', models.CharField(max_length=100, unique=True)),
            ],
            options={
                'verbose_name': 'StudentIn',
                'verbose_name_plural': 'StudentInnen',
                'ordering': ['nickname'],
            },
        ),
        migrations.CreateModel(
            name='Gender',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('bezeichnung', models.CharField(max_length=100)),
                ('endung', models.CharField(blank=True, max_length=8)),
            ],
            options={
                'verbose_name': 'Gender/Geschlecht',
                'verbose_name_plural': 'Gender/Geschlechter',
                'ordering': ['bezeichnung'],
            },
        ),
        migrations.CreateModel(
            name='Studienabschnitt',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('bezeichnung', models.CharField(max_length=100)),
                ('sortierung', models.PositiveSmallIntegerField()),
            ],
            options={
                'verbose_name': 'Studienabschnitt/Semester',
                'verbose_name_plural': 'Studienabschnitte/Semester',
                'ordering': ['sortierung'],
            },
        ),
        migrations.CreateModel(
            name='Studiengang',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('bezeichnung', models.CharField(max_length=100)),
                ('studienabschnitt', models.ManyToManyField(blank=True, related_name='studiengang', to='fsmedhro_core.Studienabschnitt')),
            ],
            options={
                'verbose_name': 'Studiengang',
                'verbose_name_plural': 'Studiengänge',
                'ordering': ['bezeichnung'],
            },
        ),
        migrations.AddField(
            model_name='fachschaftuser',
            name='gender',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='fsmedhro_core.Gender'),
        ),
        migrations.AddField(
            model_name='fachschaftuser',
            name='studienabschnitt',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='fsmedhro_core.Studienabschnitt'),
        ),
        migrations.AddField(
            model_name='fachschaftuser',
            name='studiengang',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='fsmedhro_core.Studiengang'),
        ),
        migrations.AddField(
            model_name='fachschaftuser',
            name='user',
            field=models.OneToOneField(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
        ),
    ]



from django.contrib.auth.models import User
from django.forms import ModelForm

from .models import FachschaftUser


class UserForm(ModelForm):
    class Meta:
        model = User
        fields = ['first_name', 'last_name', 'email']


class FachschaftUserForm(ModelForm):
    class Meta:
        model = FachschaftUser
        fields = ['nickname', 'studiengang', 'studienabschnitt', 'gender']



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-fsmedhro-core',
    version='0.2.1',
    packages=find_packages(),
    include_package_data=True,
    description='Basisapp für die Webseite des FSR Medizin der Uni Rostock',
    long_description=README,
    license='MIT License',
    url='https://github.com/hutchison/django-fsmedhro-core',
    install_requires=[
        'Django>=2.1',
        'django-crispy-forms',
    ],
    python_requires=">=3.5",
    author='Martin Darmüntzel',
    author_email='martin@trivialanalog.de',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.1',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
